@@ -1,6 +1,12 @@
 package cli
 
 import (
+	"sort"
+	"strings"
+
+	"github.com/posener/complete"
+	empty "google.golang.org/protobuf/types/known/emptypb"
+
 	"github.com/hashicorp/waypoint-plugin-sdk/terminal"
 	"github.com/hashicorp/waypoint/internal/clierrors"
 	"github.com/hashicorp/waypoint/internal/installutil"
@@ -9,10 +15,6 @@ import (
 	"github.com/hashicorp/waypoint/pkg/server"
 	pb "github.com/hashicorp/waypoint/pkg/server/gen"
 	"github.com/hashicorp/waypoint/pkg/serverconfig"
-	"github.com/posener/complete"
-	empty "google.golang.org/protobuf/types/known/emptypb"
-	"sort"
-	"strings"
 )
 
 type RunnerInstallCommand struct {
@@ -20,7 +22,7 @@ type RunnerInstallCommand struct {
 
 	platform              []string `hcl:"platform,optional"`
 	skipAdopt             bool     `hcl:"skip_adopt,optional"`
-	serverUrl             string   `hcl:"server_url,required"`
+	serverUrl             string   `hcl:"server_url"`
 	id                    string   `hcl:"id,optional"`
 	runnerProfileOdrImage string   `hcl:"odr_image,optional"`
 	serverTls             bool     `hcl:"server_tls,optional"`
@@ -64,7 +66,7 @@ func (c *RunnerInstallCommand) Flags() *flag.Sets {
 		f.StringVar(&flag.StringVar{
 			Name:    "odr-image",
 			Usage:   "Docker image for the on-demand runners.",
-			Default: "hashicorp/waypoint-odr:latest",
+			Default: installutil.DefaultRunnerImage,
 			Target:  &c.runnerProfileOdrImage,
 		})
 
@@ -119,6 +121,15 @@ Usage: waypoint runner install [options]
   the current Waypoint context. It will adopt the runner after installation, 
   unless the '-skip-adopt' flag is set to true.
 
+  To further customize the runner installation, you may pass advanced flag
+  options specified in the documentation for the 'runner agent' command. To set
+  these values, include a '--' after the full argument list for 'install',
+  followed by these advanced flag options. As an example, to set a label k/v
+  on the runner profile that is generated as part of adopting a runner during
+  the install, the command would be:
+
+    waypoint runner install -server-addr=localhost:9701 -server-tls-skip-verify -- -label=environment=primary
+
 ` + c.Flags().Help())
 }
 
@@ -160,7 +171,7 @@ func (c *RunnerInstallCommand) Run(args []string) int {
 	p, ok := runnerinstall.Platforms[strings.ToLower(platform[0])]
 	if !ok {
 		c.ui.Output(
-			"Error installing server into %q: unsupported platform",
+			"Error installing runner into %q: unsupported platform",
 			platform[0],
 			terminal.WithErrorStyle(),
 		)
@@ -202,6 +213,41 @@ func (c *RunnerInstallCommand) Run(args []string) int {
 		}
 	}
 
+	// collect any args after a `--` break to pass forward as secondary flags
+	var secondaryArgs []string
+	for i, f := range args {
+		if f == "--" {
+			secondaryArgs = args[(i + 1):]
+			break
+		}
+	}
+
+	// This loop is used to parse any arguments supplied after `--` for label flags so that we can
+	// apply them to our runner profile as target labels later
+	targetLabels := make(map[string]string)
+	for i, arg := range secondaryArgs {
+		// A label flag can be either `-label=key=value` or `-label key=value`
+		// so we need to parse for both cases
+		if strings.Contains(arg, "-label=") {
+			kv := strings.Split(strings.TrimPrefix(arg, "-label="), "=")
+			targetLabels[kv[0]] = kv[1]
+		} else if strings.Contains(arg, "-label") {
+			// If -label is the final argument and there is no KV pair following it, we don't attempt to parse
+			if i+1 < len(secondaryArgs) {
+				// We get the next argument because if it's space delimited, the KV pair
+				// should be the next positional argument
+				kvPair := secondaryArgs[i+1]
+				// If there's no "=", then we skip the argument because it's not a KV pair
+				if !strings.Contains(kvPair, "=") {
+					continue
+				} else {
+					kv := strings.Split(strings.TrimPrefix(secondaryArgs[i+1], "-label="), "=")
+					targetLabels[kv[0]] = kv[1]
+				}
+			}
+		}
+	}
+
 	s = sg.Add("Installing runner...")
 	err = p.Install(ctx, &runnerinstall.InstallOpts{
 		Log:        log,
@@ -214,7 +260,8 @@ func (c *RunnerInstallCommand) Run(args []string) int {
 			TlsSkipVerify: c.serverTlsSkipVerify,
 			RequireAuth:   c.serverRequireAuth,
 		},
-		Id: id,
+		Id:               id,
+		RunnerAgentFlags: secondaryArgs,
 	})
 	if err != nil {
 		c.ui.Output("Error installing runner: %s", clierrors.Humanize(err),
@@ -236,21 +283,37 @@ func (c *RunnerInstallCommand) Run(args []string) int {
 		}
 
 		// Creating a new runner profile for the newly adopted runner
+		var odrConfig *pb.OnDemandRunnerConfig
 		s = sg.Add("Creating runner profile and targeting runner %s", strings.ToUpper(id))
-		runnerProfile, err := client.UpsertOnDemandRunnerConfig(ctx, &pb.UpsertOnDemandRunnerConfigRequest{
-			Config: &pb.OnDemandRunnerConfig{
-				Name: platform[0] + "-" + strings.ToUpper(id),
-				TargetRunner: &pb.Ref_Runner{
-					Target: &pb.Ref_Runner_Id{
-						Id: &pb.Ref_RunnerId{
-							Id: strings.ToUpper(id),
-						},
-					},
-				},
+		if odc, ok := p.(installutil.OnDemandRunnerConfigProvider); ok {
+			odrConfig = odc.OnDemandRunnerConfig()
+			odrConfig.Name = odrConfig.Name + "-" + strings.ToUpper(id)
+		} else {
+			odrConfig = &pb.OnDemandRunnerConfig{
+				Name:       platform[0] + "-" + strings.ToUpper(id),
 				OciUrl:     c.runnerProfileOdrImage,
 				PluginType: platform[0],
-			},
-		})
+			}
+		}
+		if len(targetLabels) != 0 {
+			odrConfig.TargetRunner = &pb.Ref_Runner{
+				Target: &pb.Ref_Runner_Labels{
+					Labels: &pb.Ref_RunnerLabels{
+						Labels: targetLabels,
+					},
+				},
+			}
+		} else {
+			odrConfig.TargetRunner = &pb.Ref_Runner{
+				Target: &pb.Ref_Runner_Id{
+					Id: &pb.Ref_RunnerId{
+						Id: strings.ToUpper(id),
+					},
+				},
+			}
+		}
+
+		runnerProfile, err := client.UpsertOnDemandRunnerConfig(ctx, &pb.UpsertOnDemandRunnerConfigRequest{Config: odrConfig})
 		if err != nil {
 			c.ui.Output("Error creating runner profile: %s", clierrors.Humanize(err),
 				terminal.WithErrorStyle(),
