@@ -12,8 +12,10 @@ import (
 	"github.com/hashicorp/hcl/v2"
 	"github.com/hashicorp/hcl/v2/hclsyntax"
 	hcljson "github.com/hashicorp/hcl/v2/json"
+
 	"github.com/hashicorp/waypoint-plugin-sdk/terminal"
 	"github.com/hashicorp/waypoint/internal/clierrors"
+	"github.com/hashicorp/waypoint/internal/installutil"
 	"github.com/hashicorp/waypoint/internal/pkg/flag"
 	pb "github.com/hashicorp/waypoint/pkg/server/gen"
 )
@@ -27,7 +29,7 @@ type RunnerProfileSetCommand struct {
 	flagEnvVars            []string
 	flagPluginType         string
 	flagPluginConfig       string
-	flagDefault            bool
+	flagDefault            *bool
 	flagTargetRunnerId     string
 	flagTargetRunnerLabels map[string]string
 }
@@ -188,7 +190,9 @@ func (c *RunnerProfileSetCommand) Run(args []string) int {
 
 	od.OciUrl = c.flagOCIUrl
 	od.EnvironmentVariables = map[string]string{}
-	od.Default = c.flagDefault
+	if c.flagDefault != nil {
+		od.Default = *c.flagDefault
+	}
 
 	if c.flagEnvVars != nil {
 		//TODO(XX): Deprecate -env-vars and this logic
@@ -253,7 +257,7 @@ func (c *RunnerProfileSetCommand) Flags() *flag.Sets {
 		f.StringVar(&flag.StringVar{
 			Name:    "oci-url",
 			Target:  &c.flagOCIUrl,
-			Default: "hashicorp/waypoint-odr:latest",
+			Default: installutil.DefaultRunnerImage,
 			Usage:   "The url for the OCI image to launch for the on-demand runner.",
 		})
 
@@ -287,10 +291,9 @@ func (c *RunnerProfileSetCommand) Flags() *flag.Sets {
 				"the environment the plugin will launch the on-demand runner in.",
 		})
 
-		f.BoolVar(&flag.BoolVar{
-			Name:    "default",
-			Target:  &c.flagDefault,
-			Default: false,
+		f.BoolPtrVar(&flag.BoolPtrVar{
+			Name:   "default",
+			Target: &c.flagDefault,
 			Usage: "Indicates that this remote runner profile should be the default for any project that doesn't " +
 				"otherwise specify its own remote runner.",
 		})
