@@ -4,7 +4,6 @@ import (
 	"context"
 	"encoding/json"
 	"fmt"
-	"github.com/hashicorp/waypoint/builtin/k8s"
 	"os"
 	"sort"
 	"strings"
@@ -17,9 +16,11 @@ import (
 
 	"github.com/hashicorp/go-version"
 	"github.com/hashicorp/waypoint-plugin-sdk/terminal"
+	"github.com/hashicorp/waypoint/builtin/k8s"
 	clientpkg "github.com/hashicorp/waypoint/internal/client"
 	"github.com/hashicorp/waypoint/internal/clierrors"
 	"github.com/hashicorp/waypoint/internal/clisnapshot"
+	"github.com/hashicorp/waypoint/internal/installutil"
 	"github.com/hashicorp/waypoint/internal/pkg/flag"
 	"github.com/hashicorp/waypoint/internal/runnerinstall"
 	"github.com/hashicorp/waypoint/internal/serverinstall"
@@ -251,6 +252,7 @@ func (c *ServerUpgradeCommand) Run(args []string) int {
 	runnerOpts := &runnerinstall.InstallOpts{
 		Log: log,
 		UI:  c.ui,
+		Id:  "static",
 	}
 
 	// Upgrade in place
@@ -409,7 +411,7 @@ func (c *ServerUpgradeCommand) upgradeRunner(
 	s.Update("Previous runner uninstalled")
 	s.Done()
 
-	if odc, ok := p.(serverinstall.OnDemandRunnerConfigProvider); ok {
+	if odc, ok := p.(installutil.OnDemandRunnerConfigProvider); ok {
 		odr := odc.OnDemandRunnerConfig()
 
 		runnerConfigName := odr.PluginType + "-bootstrap-profile"
@@ -429,7 +431,7 @@ func (c *ServerUpgradeCommand) upgradeRunner(
 		} else {
 			ociUrl := odr.OciUrl
 			if ociUrl == "" {
-				ociUrl = "hashicorp/waypoint-odr:latest"
+				ociUrl = installutil.DefaultRunnerImage
 			}
 			odr = &pb.OnDemandRunnerConfig{
 				Id:                   oldRunnerConfig.Config.Id,
@@ -473,6 +475,9 @@ func (c *ServerUpgradeCommand) upgradeRunner(
 					case "kubernetes":
 						// attempt to parse the runner profile config into the correct task launcher config struct
 						var result *k8s.TaskLauncherConfig
+						// NOTE(briancain): This is here due to a k8s task plugin bug. When
+						// we attempt to upgrade if we detect the previous mistake we warn
+						// users that certain key values in their plugin config are wrong.
 						if cfg.ConfigFormat == pb.Hcl_JSON {
 							err = json.Unmarshal(cfg.PluginConfig, result)
 							if err != nil {
@@ -506,15 +511,17 @@ func (c *ServerUpgradeCommand) upgradeRunner(
 				}
 			}
 
-			c.ui.Output("")
-			c.ui.Output(runnerMultiDefault, strings.Join(runnerDefaultNames[:], "\n"), strings.Join(runnerUnsetStr[:], "\n"), terminal.WithWarningStyle())
-			c.ui.Output("")
+			if len(runnerDefaultNames) > 0 {
+				c.ui.Output("")
+				c.ui.Output(runnerMultiDefault, strings.Join(runnerDefaultNames[:], "\n"), strings.Join(runnerUnsetStr[:], "\n"), terminal.WithWarningStyle())
+				c.ui.Output("")
+			}
 		}
 
 		// TODO(mitchellh): This creates a new auth token for the new runner.
 		// In the future, we need to invalidate the old token. We don't have
 		// the functionality to do this today.
-		return installRunner(ctx, installOpts.Log, client, c.ui, p, advertiseAddr, odr)
+		return installRunner(ctx, installOpts.Log, client, c.ui, p, advertiseAddr, odr, true)
 	}
 	return 0
 }
