@@ -2,7 +2,9 @@ package installutil
 
 import (
 	"fmt"
+
 	"github.com/distribution/distribution/v3/reference"
+	pb "github.com/hashicorp/waypoint/pkg/server/gen"
 )
 
 // DefaultODRImage returns the default Waypoint ODR image based on the
@@ -28,3 +30,17 @@ func DefaultODRImage(serverImage string) (string, error) {
 }
 
 const DefaultServerImage = "hashicorp/waypoint:latest"
+
+// When we have a serverImage value to give to DefaultODRImage,
+// we should use that. When we don't, we can use this value
+const DefaultRunnerImage = "hashicorp/waypoint-odr:latest"
+
+func DefaultRunnerName(id string) string {
+	return "waypoint-" + id + "-runner"
+}
+
+// An optional interface that the installer can implement to request
+// an ondemand runner be registered.
+type OnDemandRunnerConfigProvider interface {
+	OnDemandRunnerConfig() *pb.OnDemandRunnerConfig
+}
