@@ -3,6 +3,8 @@ package runnerinstall
 import (
 	"context"
 	"fmt"
+	"github.com/hashicorp/waypoint/internal/installutil"
+	pb "github.com/hashicorp/waypoint/pkg/server/gen"
 	"os"
 	"time"
 
@@ -106,7 +108,7 @@ func (i *DockerRunnerInstaller) Install(ctx context.Context, opts *InstallOpts)
 		User:         "root",
 		Image:        runnerImage,
 		Env:          opts.AdvertiseClient.Env(),
-		Cmd:          []string{"runner", "agent", "-id=" + opts.Id, "-cookie=" + opts.Cookie, "-vv"},
+		Cmd:          append([]string{"runner", "agent", "-id=" + opts.Id, "-cookie=" + opts.Cookie, "-vv"}, opts.RunnerAgentFlags...),
 		Labels: map[string]string{
 			"waypoint-type": "runner",
 		},
@@ -166,7 +168,7 @@ func (d DockerRunnerInstaller) Uninstall(ctx context.Context, opts *InstallOpts)
 
 	s.Update("Finding runner container")
 	containerNames := []string{
-		defaultRunnerName(opts.Id),
+		installutil.DefaultRunnerName(opts.Id),
 		DefaultRunnerTagName,
 	}
 	var foundContainer types.Container
@@ -209,5 +211,14 @@ func (d DockerRunnerInstaller) Uninstall(ctx context.Context, opts *InstallOpts)
 	return nil
 }
 
-func (d DockerRunnerInstaller) UninstallFlags(set *flag.Set) {
+func (d DockerRunnerInstaller) UninstallFlags(set *flag.Set) {}
+
+func (i *DockerRunnerInstaller) OnDemandRunnerConfig() *pb.OnDemandRunnerConfig {
+	// TODO: Add options for plugin config here
+	return &pb.OnDemandRunnerConfig{
+		Name:       "docker",
+		OciUrl:     i.Config.RunnerImage,
+		PluginType: "docker",
+		Default:    false,
+	}
 }
