@@ -2,11 +2,17 @@ package runnerinstall
 
 import (
 	"context"
+	"encoding/json"
+	"errors"
 	"fmt"
+	pb "github.com/hashicorp/waypoint/pkg/server/gen"
 	"strconv"
 	"strings"
 	"time"
 
+	"github.com/hashicorp/waypoint/internal/clierrors"
+	"github.com/hashicorp/waypoint/internal/installutil"
+
 	"github.com/aws/aws-sdk-go/aws"
 	"github.com/aws/aws-sdk-go/aws/session"
 	"github.com/aws/aws-sdk-go/service/ec2"
@@ -15,7 +21,7 @@ import (
 	"github.com/hashicorp/go-hclog"
 	"github.com/hashicorp/waypoint-plugin-sdk/terminal"
 	"github.com/hashicorp/waypoint/builtin/aws/utils"
-	installutil "github.com/hashicorp/waypoint/internal/installutil/aws"
+	awsinstallutil "github.com/hashicorp/waypoint/internal/installutil/aws"
 	"github.com/hashicorp/waypoint/internal/pkg/flag"
 	"github.com/hashicorp/waypoint/pkg/serverconfig"
 )
@@ -101,17 +107,18 @@ const odrRolePolicy = `{
 }`
 
 type ECSRunnerInstaller struct {
-	Config EcsConfig
+	Config  EcsConfig
+	netInfo *awsinstallutil.NetworkInformation
 }
 
 type EcsConfig struct {
 	Region            string   `hcl:"region,required"`
+	Cluster           string   `hcl:"cluster,required"`
 	ExecutionRoleName string   `hcl:"execution_role_name,optional"`
 	TaskRoleName      string   `hcl:"task_role_name,optional"`
 	CPU               string   `hcl:"runner_cpu,optional"`
 	Memory            string   `hcl:"memory_cpu,optional"`
 	RunnerImage       string   `hcl:"runner_image,optional"`
-	Cluster           string   `hcl:"cluster,optional"`
 	Subnets           []string `hcl:"subnets,optional"`
 }
 
@@ -119,6 +126,12 @@ func (i *ECSRunnerInstaller) Install(ctx context.Context, opts *InstallOpts) err
 	ui := opts.UI
 	log := opts.Log
 
+	if i.Config.Cluster == "" {
+		return errors.New("cluster name not specified")
+	} else if i.Config.Region == "" {
+		return errors.New("region not specified")
+	}
+
 	sess, err := utils.GetSession(&utils.SessionConfig{
 		Region: i.Config.Region,
 		Logger: log,
@@ -128,14 +141,14 @@ func (i *ECSRunnerInstaller) Install(ctx context.Context, opts *InstallOpts) err
 	}
 
 	var (
-		efsInfo       *installutil.EfsInformation
+		efsInfo       *awsinstallutil.EfsInformation
 		logGroup      string
 		executionRole string
-		netInfo       *installutil.NetworkInformation
+		netInfo       *awsinstallutil.NetworkInformation
 		taskRole      string
 		runSvcArn     *string
 	)
-	lf := &installutil.Lifecycle{
+	lf := &awsinstallutil.Lifecycle{
 		Init: func(ui terminal.UI) error {
 			sess, err = utils.GetSession(&utils.SessionConfig{
 				Region: i.Config.Region,
@@ -145,15 +158,16 @@ func (i *ECSRunnerInstaller) Install(ctx context.Context, opts *InstallOpts) err
 				return err
 			}
 
-			if netInfo, err = installutil.SetupNetworking(ctx, ui, sess, i.Config.Subnets); err != nil {
+			if netInfo, err = awsinstallutil.SetupNetworking(ctx, ui, sess, i.Config.Subnets); err != nil {
+				return err
+			}
+			i.netInfo = netInfo
+
+			if efsInfo, err = awsinstallutil.SetupEFS(ctx, ui, sess, netInfo); err != nil {
 				return err
 			}
 
-			if efsInfo, err = installutil.SetupEFS(ctx, ui, sess, netInfo); err != nil {
-				return err
-			}
-
-			if executionRole, err = installutil.SetupExecutionRole(ctx, ui, log, sess, i.Config.ExecutionRoleName); err != nil {
+			if executionRole, err = awsinstallutil.SetupExecutionRole(ctx, ui, log, sess, i.Config.ExecutionRoleName); err != nil {
 				return err
 			}
 
@@ -162,7 +176,7 @@ func (i *ECSRunnerInstaller) Install(ctx context.Context, opts *InstallOpts) err
 				return err
 			}
 
-			logGroup, err = installutil.SetupLogs(ctx, ui, log, sess, defaultRunnerLogGroup)
+			logGroup, err = awsinstallutil.SetupLogs(ctx, ui, log, sess, defaultRunnerLogGroup)
 			if err != nil {
 				return err
 			}
@@ -174,6 +188,7 @@ func (i *ECSRunnerInstaller) Install(ctx context.Context, opts *InstallOpts) err
 			runSvcArn, err = launchRunner(
 				ctx, ui, log, sess,
 				opts.AdvertiseClient.Env(),
+				opts.RunnerAgentFlags,
 				executionRole,
 				taskRole,
 				logGroup,
@@ -265,7 +280,7 @@ func (i *ECSRunnerInstaller) Uninstall(ctx context.Context, opts *InstallOpts) e
 	sg := ui.StepGroup()
 	defer sg.Wait()
 
-	s := sg.Add("Uninstalling Runner...")
+	s := sg.Add("Finding runner in ECS services...")
 	defer func() { s.Abort() }()
 
 	sess, err := utils.GetSession(&utils.SessionConfig{
@@ -280,60 +295,46 @@ func (i *ECSRunnerInstaller) Uninstall(ctx context.Context, opts *InstallOpts) e
 	// We check for the serviceName before v0.9 and v0.9+
 	ecsSvc := ecs.New(sess)
 	serviceNames := []string{
-		defaultRunnerName(opts.Id),
 		DefaultRunnerTagName,
+		installutil.DefaultRunnerName(opts.Id),
 	}
-	var foundService *ecs.Service
-	var services *ecs.DescribeServicesOutput
-	for _, serviceName := range serviceNames {
-		ss, err := ecsSvc.DescribeServices(&ecs.DescribeServicesInput{
-			Cluster:  aws.String(i.Config.Cluster),
-			Services: []*string{aws.String(serviceName)},
+
+	services, err := awsinstallutil.FindServices(serviceNames, ecsSvc, i.Config.Cluster)
+	if err != nil {
+		log.Debug("Unable to find desired runner; %s", err)
+		ui.Output("Could not get list of ECS services: %s", clierrors.Humanize(err), terminal.WithErrorStyle())
+		return err
+	}
+
+	for _, service := range services {
+		// Delete associated runner service and tasks
+		// This does not remove the security group since it may be in use by other
+		// runners/waypoint infrastructure.
+		s.Update("Deleting runner service")
+		_, err = ecsSvc.DeleteService(&ecs.DeleteServiceInput{
+			Service: service.ServiceArn,
+			Force:   aws.Bool(true),
+			Cluster: service.ClusterArn,
 		})
-		services = ss
 		if err != nil {
-			s.Update("Could not get list of ECS services")
+			log.Debug("error deleting runner service: %s", err)
+			ui.Output("Error Deleting Runner service: %s", clierrors.Humanize(err), terminal.WithErrorStyle())
 			return err
 		}
-		if ss != nil && len(ss.Services) > 0 {
-			foundService = ss.Services[0]
-			if len(ss.Services) != 1 {
-				log.Debug("Unable to uninstall runner; expected 1 runner service named %s, found %d", serviceName, len(ss.Services))
-				return fmt.Errorf("expected 1 runner service named %s, found %d", serviceName, len(ss.Services))
-			}
-			break
+
+		s.Update("Waiting for runner service to be inactive")
+		err = ecsSvc.WaitUntilServicesInactive(&ecs.DescribeServicesInput{
+			Cluster:  service.ClusterArn,
+			Services: []*string{service.ServiceArn},
+		})
+		if err != nil {
+			log.Debug("error waint for runner to deactivate: %s", err)
+			ui.Output("Error waiting for Runner service to deactivate: %s", clierrors.Humanize(err), terminal.WithErrorStyle())
+			return err
 		}
-	}
-	if len(services.Failures) > 0 {
-		return fmt.Errorf("could not find runner named %q or %q, service is %q", serviceNames[0], serviceNames[1], *services.Failures[0].Reason)
-	}
-	clusterArn := foundService.ClusterArn
 
-	// Delete associated runner service and tasks
-	// This does not remove the security group since it may be in use by other
-	// runners/waypoint infrastructure.
-	s.Update("Deleting runner service")
-	_, err = ecsSvc.DeleteService(&ecs.DeleteServiceInput{
-		Service: services.Services[0].ServiceArn,
-		Force:   aws.Bool(true),
-		Cluster: clusterArn,
-	})
-	if err != nil {
-		s.Update("Unable to delete runner service.")
-		return err
+		s.Update("Runner uninstalled")
 	}
-
-	s.Update("Waiting for runner service to be inactive")
-	err = ecsSvc.WaitUntilServicesInactive(&ecs.DescribeServicesInput{
-		Cluster:  clusterArn,
-		Services: []*string{services.Services[0].ServiceArn},
-	})
-	if err != nil {
-		s.Update("Unable to verify runner uninstalled", len(services.Services))
-		return err
-	}
-
-	s.Update("Runner uninstalled")
 	s.Done()
 	return nil
 }
@@ -351,7 +352,6 @@ func (i *ECSRunnerInstaller) UninstallFlags(set *flag.Set) {
 		Default: "waypoint-server",
 		Usage:   "The name of the ECS Cluster to install the Waypoint runner into.",
 	})
-
 }
 
 func launchRunner(
@@ -359,12 +359,11 @@ func launchRunner(
 	ui terminal.UI,
 	log hclog.Logger,
 	sess *session.Session,
-	env []string,
+	env, extraArgs []string,
 	executionRoleArn, taskRoleArn, logGroup, region, cpu, memory, runnerImage, cluster, cookie, id string,
-	netInfo *installutil.NetworkInformation,
-	efsInfo *installutil.EfsInformation,
+	netInfo *awsinstallutil.NetworkInformation,
+	efsInfo *awsinstallutil.EfsInformation,
 ) (*string, error) {
-
 	sg := ui.StepGroup()
 	defer sg.Wait()
 
@@ -396,9 +395,15 @@ func launchRunner(
 			Value: aws.String(value),
 		})
 	}
+
+	var args []*string
+	for _, arg := range extraArgs {
+		args = append(args, aws.String(arg))
+	}
+
 	def := ecs.ContainerDefinition{
 		Essential: aws.Bool(true),
-		Command: []*string{
+		Command: append([]*string{
 			aws.String("runner"),
 			aws.String("agent"),
 			aws.String("-id=" + id),
@@ -406,7 +411,7 @@ func launchRunner(
 			aws.String("-cookie=" + cookie),
 			aws.String("-state-dir=/data/runner"),
 			aws.String("-vv"),
-		},
+		}, args...),
 		Name:  aws.String(runnerName),
 		Image: aws.String(runnerImage),
 		PortMappings: []*ecs.PortMapping{
@@ -481,7 +486,7 @@ func launchRunner(
 		Filters: []*ec2.Filter{
 			{
 				Name:   aws.String("group-name"),
-				Values: []*string{aws.String(installutil.DefaultSecurityGroupName)},
+				Values: []*string{aws.String(awsinstallutil.DefaultSecurityGroupName)},
 			},
 		},
 	})
@@ -492,9 +497,9 @@ func launchRunner(
 	var groupId *string
 	if len(dsg.SecurityGroups) != 0 {
 		groupId = dsg.SecurityGroups[0].GroupId
-		s.Update("Using existing security group: %s", installutil.DefaultSecurityGroupName)
+		s.Update("Using existing security group: %s", awsinstallutil.DefaultSecurityGroupName)
 	} else {
-		return nil, fmt.Errorf("could not find security group (%s)", installutil.DefaultSecurityGroupName)
+		return nil, fmt.Errorf("could not find security group (%s)", awsinstallutil.DefaultSecurityGroupName)
 	}
 
 	// Check for details of possibly existing cluster `waypoint-server`
@@ -502,7 +507,7 @@ func launchRunner(
 	// query what subnets and vpc information from the server service
 	services, err := ecsSvc.DescribeServices(&ecs.DescribeServicesInput{
 		Cluster:  aws.String(cluster),
-		Services: []*string{aws.String(installutil.ServerName)},
+		Services: []*string{aws.String(awsinstallutil.ServerName)},
 	})
 	if err != nil {
 		return nil, err
@@ -526,7 +531,7 @@ func launchRunner(
 		Cluster:              clusterArn,
 		DesiredCount:         aws.Int64(1),
 		LaunchType:           aws.String(defaultTaskRuntime),
-		ServiceName:          aws.String(defaultRunnerName(id)),
+		ServiceName:          aws.String(installutil.DefaultRunnerName(id)),
 		EnableECSManagedTags: aws.Bool(true),
 		TaskDefinition:       aws.String(taskDefArn),
 		NetworkConfiguration: &ecs.NetworkConfiguration{
@@ -549,7 +554,7 @@ func launchRunner(
 	}
 
 	s.Update("Creating ECS Service (%s)", DefaultRunnerTagName)
-	svc, err := installutil.CreateService(createServiceInput, ecsSvc)
+	svc, err := awsinstallutil.CreateService(createServiceInput, ecsSvc)
 	if err != nil {
 		return nil, err
 	}
@@ -639,7 +644,7 @@ func (i *ECSRunnerInstaller) setupTaskRole(
 	s.Update("Creating IAM task role: %s", roleName)
 
 	input := &iam.CreateRoleInput{
-		AssumeRolePolicyDocument: aws.String(installutil.RolePolicy),
+		AssumeRolePolicyDocument: aws.String(awsinstallutil.RolePolicy),
 		Path:                     aws.String("/"),
 		RoleName:                 aws.String(roleName),
 		Tags: []*iam.Tag{
@@ -690,6 +695,48 @@ func (i *ECSRunnerInstaller) setupTaskRole(
 	return roleArn, nil
 }
 
+// OnDemandRunnerConfig implements OnDemandRunnerConfigProvider
+func (i *ECSRunnerInstaller) OnDemandRunnerConfig() *pb.OnDemandRunnerConfig {
+	// Generate some configuration. Some of the OnDemand configurations have
+	// defaults so we should be fine to directly use them
+	cfgMap := map[string]interface{}{
+		"log_group":           defaultRunnerLogGroup,
+		"execution_role_name": i.Config.ExecutionRoleName,
+		"task_role_name":      i.Config.TaskRoleName,
+		"cluster":             i.Config.Cluster,
+		"region":              i.Config.Region,
+		"odr_cpu":             i.Config.CPU,
+		"odr_memory":          i.Config.Memory,
+	}
+
+	if i.netInfo != nil {
+		var subnets []string
+		for _, s := range i.netInfo.Subnets {
+			subnets = append(subnets, *s)
+		}
+		cfgMap["subnets"] = strings.Join(subnets, ",")
+		cfgMap["security_group_id"] = i.netInfo.SgID
+	}
+
+	// Marshal our config
+	cfgJson, err := json.MarshalIndent(cfgMap, "", "\t")
+	if err != nil {
+		// This shouldn't happen cause we control our input. If it does,
+		// just panic cause this will be in a `server install` CLI and
+		// we want the user to report a bug.
+		panic(err)
+	}
+
+	return &pb.OnDemandRunnerConfig{
+		Name:         "ecs",
+		OciUrl:       i.Config.RunnerImage,
+		PluginType:   "aws-ecs",
+		Default:      false,
+		PluginConfig: cfgJson,
+		ConfigFormat: pb.Hcl_JSON,
+	}
+}
+
 type Logging struct {
 	CreateGroup      bool   `hcl:"create_group,optional"`
 	StreamPrefix     string `hcl:"stream_prefix,optional"`
