@@ -3,13 +3,15 @@ package serverinstall
 import (
 	"context"
 	json "encoding/json"
+	"errors"
 	"fmt"
-	"github.com/hashicorp/waypoint/internal/installutil"
 	"os"
 	"strconv"
 	"strings"
 	"time"
 
+	"github.com/hashicorp/waypoint/internal/installutil"
+
 	"github.com/hashicorp/waypoint/internal/installutil/nomad"
 	"github.com/hashicorp/waypoint/internal/runnerinstall"
 
@@ -37,6 +39,9 @@ type nomadConfig struct {
 	namespace          string            `hcl:"namespace,optional"`
 	serviceAnnotations map[string]string `hcl:"service_annotations,optional"`
 
+	serviceProvider          string   `hcl:"service_provider,optional"`
+	serviceUITags            []string `hcl:"service_ui_tags:optional"`
+	serviceBackendTags       []string `hcl:"service_backend_tags:optional"`
 	consulService            bool     `hcl:"consul_service,optional"`
 	consulServiceUITags      []string `hcl:"consul_service_ui_tags:optional"`
 	consulServiceBackendTags []string `hcl:"consul_service_backend_tags:optional"`
@@ -67,6 +72,7 @@ type nomadConfig struct {
 	csiTopologies        map[string]string `hcl:"nomad_csi_topologies,optional"`
 	csiSecrets           map[string]string `hcl:"nomad_csi_secrets,optional"`
 	csiParams            map[string]string `hcl:"csi_parameters,optional"`
+	csiVolume            string            `hcl:"nomad_csi_volume,optional"`
 	nomadHost            string            `hcl:"nomad_host,optional"`
 
 	runnerResourcesCPU         string `hcl:"runner_resources_cpu,optional"`
@@ -75,6 +81,7 @@ type nomadConfig struct {
 	runnerCsiVolumeProvider    string `hcl:"runner_csi_volume_provider,optional"`
 	runnerCsiVolumeCapacityMin int64  `hcl:"runner_csi_volume_capacity_min,optional"`
 	runnerCsiVolumeCapacityMax int64  `hcl:"runner_csi_volume_capacity_max,optional"`
+	runnerCsiVolume            string `hcl:"runner_csi_volume,optional"`
 }
 
 var (
@@ -90,13 +97,13 @@ var (
 	defaultCSIVolumeMountFS = "xfs"
 
 	// Defaults to use for setting up Consul
-	defaultConsulServiceTag       = "waypoint"
+	defaultServiceTag             = "waypoint"
 	defaultConsulDatacenter       = "dc1"
 	defaultConsulDomain           = "consul"
-	waypointConsulBackendName     = "waypoint-server"
-	waypointConsulUIName          = "waypoint-ui"
+	waypointBackendServiceName    = "waypoint-server"
+	waypointUIServiceName         = "waypoint-ui"
 	defaultWaypointConsulHostname = fmt.Sprintf("%s.%s.service.%s.%s",
-		defaultConsulServiceTag, waypointConsulBackendName, defaultConsulDatacenter, defaultConsulDomain)
+		defaultServiceTag, waypointBackendServiceName, defaultConsulDatacenter, defaultConsulDomain)
 
 	defaultNomadHost = "http://localhost:4646"
 )
@@ -106,7 +113,7 @@ var (
 func (i *NomadInstaller) Install(
 	ctx context.Context,
 	opts *InstallOpts,
-) (*InstallResults, error) {
+) (*InstallResults, string, error) {
 	ui := opts.UI
 
 	sg := ui.StepGroup()
@@ -118,7 +125,7 @@ func (i *NomadInstaller) Install(
 	// Build api client from environment
 	client, err := api.NewClient(api.DefaultConfig())
 	if err != nil {
-		return nil, err
+		return nil, "", err
 	}
 
 	s.Update("Checking for existing Waypoint server...")
@@ -126,14 +133,14 @@ func (i *NomadInstaller) Install(
 	// Check if waypoint-server has already been deployed
 	jobs, _, err := client.Jobs().PrefixList(serverName)
 	if err != nil {
-		return nil, err
+		return nil, "", err
 	}
 
 	var serverDetected bool
 	for _, j := range jobs {
 		if j.Name == serverName {
 			if j.Status != "running" {
-				return nil, fmt.Errorf("waypoint-server job found but not running")
+				return nil, "", fmt.Errorf("waypoint-server job found but not running")
 			}
 			serverDetected = true
 			break
@@ -144,7 +151,7 @@ func (i *NomadInstaller) Install(
 		var err error
 		i.config.odrImage, err = installutil.DefaultODRImage(i.config.serverImage)
 		if err != nil {
-			return nil, err
+			return nil, "", err
 		}
 	}
 
@@ -166,7 +173,7 @@ func (i *NomadInstaller) Install(
 	if serverDetected {
 		allocs, _, err := client.Jobs().Allocations(serverName, false, nil)
 		if err != nil {
-			return nil, err
+			return nil, "", err
 		}
 
 		var activeAllocs []*api.AllocationListStub
@@ -176,11 +183,11 @@ func (i *NomadInstaller) Install(
 			}
 		}
 		if len(allocs) == 0 || len(activeAllocs) == 0 {
-			return nil, fmt.Errorf("waypoint-server job found but no running allocations available")
+			return nil, "", fmt.Errorf("waypoint-server job found but no running allocations available")
 		}
 		serverAddr, err := getAddrFromAllocID(allocs[0].ID, client)
 		if err != nil {
-			return nil, err
+			return nil, "", err
 		}
 
 		s.Update("Detected existing Waypoint server")
@@ -194,14 +201,14 @@ func (i *NomadInstaller) Install(
 			Context:       &clicfg,
 			AdvertiseAddr: &addr,
 			HTTPAddr:      httpAddr,
-		}, nil
+		}, "", nil
 	}
 
 	if i.config.csiVolumeProvider == "" && i.config.hostVolume == "" {
-		return nil, fmt.Errorf("please include '-nomad-csi-volume-provider' or '-nomad-host-volume'")
+		return nil, "", fmt.Errorf("please include '-nomad-csi-volume-provider' or '-nomad-host-volume'")
 	} else if i.config.csiVolumeProvider != "" {
 		if i.config.hostVolume != "" {
-			return nil, fmt.Errorf("choose either CSI or host volume, not both")
+			return nil, "", fmt.Errorf("choose either CSI or host volume, not both")
 		}
 
 		s.Update("Creating persistent volume")
@@ -209,7 +216,7 @@ func (i *NomadInstaller) Install(
 			ctx,
 			client,
 			"waypoint-server",
-			"waypoint-server",
+			i.config.csiVolume,
 			i.config.csiPluginId,
 			i.config.csiVolumeProvider,
 			i.config.csiFS,
@@ -220,7 +227,7 @@ func (i *NomadInstaller) Install(
 			i.config.csiSecrets,
 		)
 		if err != nil {
-			return nil, status.Errorf(codes.Internal, "Failed creating Nomad persistent volume: %s", err)
+			return nil, "", status.Errorf(codes.Internal, "Failed creating Nomad persistent volume: %s", err)
 		}
 		s.Update("Persistent volume created!")
 		s.Status(terminal.StatusOK)
@@ -228,50 +235,18 @@ func (i *NomadInstaller) Install(
 	}
 
 	s.Update("Installing Waypoint server to Nomad")
-	allocID, err := nomad.RunJob(ctx, s, client, waypointNomadJob(i.config, opts.ServerRunFlags), i.config.policyOverride)
+	allocID, err := nomad.RunJob(ctx, s, client, waypointNomadJob(i.config, opts.ServerRunFlags, false), i.config.policyOverride)
 	if err != nil {
-		return nil, err
+		return nil, "", err
 	}
 
 	// If a Consul service was requested, set the consul DNS hostname rather
 	// than the direct static IP for the CLI context and server config. Otherwise
 	// if Nomad restarts the server allocation, a new IP will be assigned and any
 	// configured clients will be invalid
-	if i.config.consulService {
-		s.Update("Configuring the server context to use Consul DNS hostname")
-		if i.config.consulDatacenter == "" {
-			i.config.consulDatacenter = defaultConsulDatacenter
-		}
-		if i.config.consulDomain == "" {
-			i.config.consulDomain = defaultConsulDomain
-		}
-
-		grpcPort, _ := strconv.Atoi(defaultGrpcPort)
-		httpPort, _ := strconv.Atoi(defaultHttpPort)
-
-		if i.config.consulServiceHostname == "" {
-			addr.Addr = fmt.Sprintf("%s.service.%s.%s:%d",
-				waypointConsulBackendName, i.config.consulDatacenter, i.config.consulDomain, grpcPort)
-			httpAddr = fmt.Sprintf("%s.service.%s.%s:%d",
-				waypointConsulUIName, i.config.consulDatacenter, i.config.consulDomain, httpPort)
-		} else {
-			addr.Addr = fmt.Sprintf("%s:%d", i.config.consulServiceHostname, grpcPort)
-			httpAddr = fmt.Sprintf("%s:%d", i.config.consulServiceHostname, httpPort)
-		}
-	} else {
-		s.Update("Configuring the server context to use the static IP address from the Nomad allocation")
-
-		serverAddr, err := getAddrFromAllocID(allocID, client)
-		if err != nil {
-			return nil, err
-		}
-		hAddr, err := getHTTPFromAllocID(allocID, client)
-		if err != nil {
-			return nil, err
-		}
-
-		httpAddr = hAddr
-		addr.Addr = serverAddr
+	httpAddr, addr.Addr, err = i.getWaypointAddress(client, allocID)
+	if err != nil {
+		return nil, "", err
 	}
 
 	clicfg = clicontext.Config{
@@ -286,26 +261,71 @@ func (i *NomadInstaller) Install(
 	s.Update("Waypoint server ready")
 	s.Done()
 
-	if i.config.consulService {
-		s = sg.Add("The CLI has been configured to automatically install a Consul service for\n" +
+	if i.config.serviceProvider == "consul" && i.config.consulService {
+		opts.UI.Output("The CLI has been configured to automatically install a Consul service for\n" +
+			"the Waypoint service backend and ui service in Nomad.")
+	} else if i.config.serviceProvider == "nomad" {
+		opts.UI.Output("The CLI has been configured to automatically install a Nomad service for\n" +
 			"the Waypoint service backend and ui service in Nomad.")
-		s.Done()
 	} else {
-		s = sg.Add(
-			" Waypoint server running on Nomad is being accessed via its allocation IP and port.\n" +
-				"This could change in the future if Nomad creates a new allocation for the Waypoint server,\n" +
-				"which would break all existing Waypoint contexts.\n\n" +
-				"It is recommended to use Consul for determining Waypoint servers IP running on Nomad rather than\n" +
-				"relying on the static IP that is initially set up for this allocation.")
-		s.Status(terminal.StatusWarn)
-		s.Done()
+		opts.UI.Output("Waypoint server running on Nomad is being accessed via its allocation IP and port.\n" +
+			"This could change in the future if Nomad creates a new allocation for the Waypoint server,\n" +
+			"which would break all existing Waypoint contexts.\n\n" +
+			"It is recommended to use Consul for determining Waypoint servers IP running on Nomad rather than\n" +
+			"relying on the static IP that is initially set up for this allocation.")
 	}
 
 	return &InstallResults{
 		Context:       &clicfg,
 		AdvertiseAddr: &addr,
 		HTTPAddr:      httpAddr,
-	}, nil
+	}, "", nil
+}
+
+func (i *NomadInstaller) getWaypointAddress(client *api.Client, allocID string) (string, string, error) {
+	if i.config.serviceProvider == "consul" && i.config.consulService {
+		if i.config.consulDatacenter == "" {
+			i.config.consulDatacenter = defaultConsulDatacenter
+		}
+		if i.config.consulDomain == "" {
+			i.config.consulDomain = defaultConsulDomain
+		}
+
+		grpcPort, _ := strconv.Atoi(defaultGrpcPort)
+		httpPort, _ := strconv.Atoi(defaultHttpPort)
+
+		if i.config.consulServiceHostname == "" {
+			return fmt.Sprintf("%s.service.%s.%s:%d",
+					waypointBackendServiceName, i.config.consulDatacenter, i.config.consulDomain, grpcPort), fmt.Sprintf("%s.service.%s.%s:%d",
+					waypointUIServiceName, i.config.consulDatacenter, i.config.consulDomain, httpPort), nil
+		} else {
+			return fmt.Sprintf("%s:%d", i.config.consulServiceHostname, grpcPort),
+				fmt.Sprintf("%s:%d", i.config.consulServiceHostname, httpPort), nil
+		}
+	} else if i.config.serviceProvider == "nomad" {
+		backendService, _, err := client.Services().Get(waypointBackendServiceName, nil)
+		if err != nil {
+			return "", "", err
+		}
+
+		uiService, _, err := client.Services().Get(waypointUIServiceName, nil)
+		if err != nil {
+			return "", "", err
+		}
+
+		return fmt.Sprintf("%s:%d", uiService[0].Address, uiService[0].Port),
+			fmt.Sprintf("%s:%d", backendService[0].Address, backendService[0].Port), nil
+	} else {
+		serverAddr, err := getAddrFromAllocID(allocID, client)
+		if err != nil {
+			return "", "", err
+		}
+		hAddr, err := getHTTPFromAllocID(allocID, client)
+		if err != nil {
+			return "", "", err
+		}
+		return hAddr, serverAddr, nil
+	}
 }
 
 // Upgrade is a method of NomadInstaller and implements the Installer interface to
@@ -319,13 +339,26 @@ func (i *NomadInstaller) Upgrade(
 	sg := ui.StepGroup()
 	defer sg.Wait()
 
-	if !i.config.consulService {
+	if i.config.serviceProvider == "none" && !i.config.consulService {
 		// By default, we don't auto-enable the consul service because prior to Waypoint
 		// version 0.6.2, we did not enable it by default.
-		sw := sg.Add("Nomad Consul Service is disabled. If you had previously enabled " +
-			"it in the last installation, please stop this upgrade and re-run with -nomad-consul-service=true.")
-		sw.Status(terminal.StatusWarn)
-		sw.Done()
+		proceed, err := opts.UI.Input(&terminal.Input{
+			Prompt: "Service discovery is disabled for the Waypoint Nomad job. If you had previously enabled " +
+				"it in the last installation, please stop this upgrade and re-run with the -nomad-service-provider flag. " +
+				"Otherwise, enter 'yes' to continue the upgrade: ",
+			Style:  "",
+			Secret: false,
+		})
+		if err != nil {
+			opts.UI.Output(
+				"Error upgrading server: %s",
+				clierrors.Humanize(err),
+				terminal.WithErrorStyle(),
+			)
+			return nil, err
+		} else if strings.ToLower(proceed) != "yes" {
+			return nil, errors.New("upgrade aborted")
+		}
 	}
 
 	s := sg.Add("Initializing Nomad client...")
@@ -402,7 +435,7 @@ func (i *NomadInstaller) Upgrade(
 	}
 
 	s = sg.Add("Upgrading Waypoint server on Nomad to %q", i.config.serverImage)
-	job := waypointNomadJob(i.config, opts.ServerRunFlags)
+	job := waypointNomadJob(i.config, opts.ServerRunFlags, true)
 	jobOpts := &api.RegisterOptions{
 		PolicyOverride: i.config.policyOverride,
 	}
@@ -477,41 +510,9 @@ func (i *NomadInstaller) Upgrade(
 	// than the direct static IP for the CLI context and server config. Otherwise
 	// if Nomad restarts the server allocation, a new IP will be assigned and any
 	// configured clients will be invalid
-	if i.config.consulService {
-		s.Update("Configuring the server context to use Consul DNS hostname")
-		if i.config.consulDatacenter == "" {
-			i.config.consulDatacenter = defaultConsulDatacenter
-		}
-		if i.config.consulDomain == "" {
-			i.config.consulDomain = defaultConsulDomain
-		}
-
-		grpcPort, _ := strconv.Atoi(defaultGrpcPort)
-		httpPort, _ := strconv.Atoi(defaultHttpPort)
-
-		if i.config.consulServiceHostname == "" {
-			addr.Addr = fmt.Sprintf("%s.service.%s.%s:%d",
-				waypointConsulBackendName, i.config.consulDatacenter, i.config.consulDomain, grpcPort)
-			httpAddr = fmt.Sprintf("%s.service.%s.%s:%d",
-				waypointConsulUIName, i.config.consulDatacenter, i.config.consulDomain, httpPort)
-		} else {
-			addr.Addr = fmt.Sprintf("%s:%d", i.config.consulServiceHostname, grpcPort)
-			httpAddr = fmt.Sprintf("%s:%d", i.config.consulServiceHostname, httpPort)
-		}
-	} else {
-		s.Update("Configuring the server context to use the static IP address from the Nomad allocation")
-
-		serverAddr, err := getAddrFromAllocID(allocID, client)
-		if err != nil {
-			return nil, err
-		}
-		hAddr, err := getHTTPFromAllocID(allocID, client)
-		if err != nil {
-			return nil, err
-		}
-
-		httpAddr = hAddr
-		addr.Addr = serverAddr
+	httpAddr, addr.Addr, err = i.getWaypointAddress(client, allocID)
+	if err != nil {
+		return nil, err
 	}
 
 	clicfg = clicontext.Config{
@@ -642,6 +643,7 @@ func (i *NomadInstaller) InstallRunner(
 			CsiExternalId:         i.config.csiExternalId,
 			CsiPluginId:           i.config.csiPluginId,
 			CsiSecrets:            i.config.csiSecrets,
+			CsiVolume:             i.config.runnerCsiVolume,
 			NomadHost:             i.config.nomadHost,
 		},
 	}
@@ -682,13 +684,23 @@ func (i *NomadInstaller) HasRunner(
 	if err != nil {
 		return false, err
 	}
+	var jobs []*api.JobListStub
 
-	jobs, _, err := client.Jobs().PrefixList(runnerName)
+	// Check for runner with job name pre-0.9
+	jobsWithOldRunnerName, _, err := client.Jobs().PrefixList(runnerName)
 	if err != nil {
 		return false, err
 	}
+	jobs = append(jobs, jobsWithOldRunnerName...)
+
+	// Check for runner with job name post-0.9
+	jobWithNewRunnerName, _, err := client.Jobs().PrefixList(runnerJobName)
+	if err != nil {
+		return false, err
+	}
+	jobs = append(jobs, jobWithNewRunnerName...)
 	for _, j := range jobs {
-		if j.Name == runnerName {
+		if j.Name == runnerJobName || j.Name == runnerName {
 			return true, nil
 		}
 	}
@@ -803,7 +815,7 @@ func (i *NomadInstaller) waitForEvaluation(
 
 // waypointNomadJob takes in a nomadConfig and returns a Nomad Job per the
 // Nomad API
-func waypointNomadJob(c nomadConfig, rawRunFlags []string) *api.Job {
+func waypointNomadJob(c nomadConfig, rawRunFlags []string, upgrade bool) *api.Job {
 	job := api.NewServiceJob(serverName, serverName, c.region, 50)
 	job.Namespace = &c.namespace
 	job.Datacenters = c.datacenters
@@ -813,9 +825,10 @@ func waypointNomadJob(c nomadConfig, rawRunFlags []string) *api.Job {
 	grpcPort, _ := strconv.Atoi(defaultGrpcPort)
 	httpPort, _ := strconv.Atoi(defaultHttpPort)
 
-	// Include services to be registered in Consul. Currently configured to happen by default
+	// Include services to be registered. Currently configured to happen with Consul by default
 	// One service added for Waypoint UI, and one for Waypoint backend port
-	if c.consulService {
+	var services []*api.Service
+	if (c.serviceProvider == "consul" && (c.consulService || upgrade)) || (c.consulService && (c.serviceProvider == "consul" || upgrade)) {
 		token := ""
 		if c.consulToken == "" {
 			token = os.Getenv("CONSUL_HTTP_TOKEN")
@@ -823,19 +836,50 @@ func waypointNomadJob(c nomadConfig, rawRunFlags []string) *api.Job {
 			token = c.consulToken
 		}
 		job.ConsulToken = &token
-		tg.Services = []*api.Service{
+
+		// if the user explicitly set tags with the Consul flags, we use those
+		// otherwise, use platform-agnostic tag flags (for backwards-compatibility)
+		uiTags := c.consulServiceUITags
+		if len(uiTags) == 0 {
+			uiTags = c.serviceUITags
+		}
+
+		backendTags := c.consulServiceBackendTags
+		if len(backendTags) == 0 {
+			backendTags = c.serviceUITags
+		}
+
+		services = []*api.Service{
 			{
-				Name:      waypointConsulUIName,
+				Name:      waypointUIServiceName,
 				PortLabel: "ui",
-				Tags:      c.consulServiceUITags,
+				Tags:      uiTags,
+				Provider:  "consul",
 			},
 			{
-				Name:      waypointConsulBackendName,
+				Name:      waypointBackendServiceName,
 				PortLabel: "server",
-				Tags:      c.consulServiceBackendTags,
+				Tags:      backendTags,
+				Provider:  "consul",
+			},
+		}
+	} else if c.serviceProvider == "nomad" {
+		services = []*api.Service{
+			{
+				Name:      waypointUIServiceName,
+				PortLabel: "ui",
+				Tags:      c.serviceUITags,
+				Provider:  "nomad",
+			},
+			{
+				Name:      waypointBackendServiceName,
+				PortLabel: "server",
+				Tags:      c.serviceBackendTags,
+				Provider:  "nomad",
 			},
 		}
 	}
+	tg.Services = services
 
 	tg.Networks = []*api.NetworkResource{
 		{
@@ -1159,6 +1203,12 @@ func (i *NomadInstaller) InstallFlags(set *flag.Set) {
 		Usage:  "Name of the CSI volume provider to use for the Waypoint runner.",
 	})
 
+	set.StringVar(&flag.StringVar{
+		Name:   "nomad-runner-csi-volume",
+		Target: &i.config.runnerCsiVolume,
+		Usage:  "The name of the volume to initialize for the Waypoint runner within the CSI provider.",
+	})
+
 	// TODO: Update default values for runner - less space is needed for runner compared to server
 	set.Int64Var(&flag.Int64Var{
 		Name:    "nomad-runner-csi-volume-capacity-min",
@@ -1181,11 +1231,33 @@ func (i *NomadInstaller) InstallFlags(set *flag.Set) {
 		Default: installutil.DefaultServerImage,
 	})
 
+	set.EnumSingleVar(&flag.EnumSingleVar{
+		Name:    "nomad-service-provider",
+		Target:  &i.config.serviceProvider,
+		Usage:   "Create service for Waypoint UI and Server in Consul.",
+		Values:  []string{"consul", "nomad", "none"},
+		Default: "consul", //default to consul for fresh installs
+	})
+
+	set.StringSliceVar(&flag.StringSliceVar{
+		Name:    "nomad-service-ui-tags",
+		Target:  &i.config.serviceUITags,
+		Usage:   "Tags for the Waypoint UI service.",
+		Default: []string{defaultServiceTag},
+	})
+
+	set.StringSliceVar(&flag.StringSliceVar{
+		Name:    "nomad-service-backend-tags",
+		Target:  &i.config.serviceBackendTags,
+		Usage:   "Tags for the Waypoint backend service.",
+		Default: []string{defaultServiceTag},
+	})
+
 	set.BoolVar(&flag.BoolVar{
 		Name:    "nomad-consul-service",
 		Target:  &i.config.consulService,
 		Usage:   "Create service for Waypoint UI and Server in Consul.",
-		Default: true, // default to true for fresh installs
+		Default: true, //default to true for fresh installs
 	})
 
 	set.StringVar(&flag.StringVar{
@@ -1197,10 +1269,9 @@ func (i *NomadInstaller) InstallFlags(set *flag.Set) {
 	})
 
 	set.StringSliceVar(&flag.StringSliceVar{
-		Name:    "nomad-consul-service-ui-tags",
-		Target:  &i.config.consulServiceUITags,
-		Usage:   "Tags for the Waypoint UI service generated in Consul.",
-		Default: []string{defaultConsulServiceTag},
+		Name:   "nomad-consul-service-ui-tags",
+		Target: &i.config.consulServiceUITags,
+		Usage:  "Tags for the Waypoint UI service generated in Consul.",
 	})
 
 	set.StringSliceVar(&flag.StringSliceVar{
@@ -1208,7 +1279,6 @@ func (i *NomadInstaller) InstallFlags(set *flag.Set) {
 		Target: &i.config.consulServiceBackendTags,
 		Usage: "Tags for the Waypoint backend service generated in Consul. The 'first' tag " +
 			"will be used when crafting the Consul DNS hostname for accessing Waypoint.",
-		Default: []string{defaultConsulServiceTag},
 	})
 
 	set.StringVar(&flag.StringVar{
@@ -1295,6 +1365,13 @@ func (i *NomadInstaller) InstallFlags(set *flag.Set) {
 		Target: &i.config.csiTopologies,
 		Usage:  "Locations from which the Nomad Volume will be accessible.",
 	})
+
+	set.StringVar(&flag.StringVar{
+		Name:    "nomad-csi-volume",
+		Target:  &i.config.csiVolume,
+		Usage:   "The name of the volume to initialize for Waypoint server within the CSI provider.",
+		Default: "waypoint-server",
+	})
 }
 
 func (i *NomadInstaller) UpgradeFlags(set *flag.Set) {
@@ -1394,6 +1471,12 @@ func (i *NomadInstaller) UpgradeFlags(set *flag.Set) {
 		Usage:  "Name of the CSI volume provider to use for the Waypoint runner.",
 	})
 
+	set.StringVar(&flag.StringVar{
+		Name:   "nomad-runner-csi-volume",
+		Target: &i.config.runnerCsiVolume,
+		Usage:  "The name of the volume to initialize for the Waypoint runner within the CSI provider.",
+	})
+
 	// TODO: Update default values for runner - less space is needed for runner compared to server
 	set.Int64Var(&flag.Int64Var{
 		Name:    "nomad-runner-csi-volume-capacity-min",
@@ -1422,6 +1505,28 @@ func (i *NomadInstaller) UpgradeFlags(set *flag.Set) {
 		Usage:  "Nomad host volume name.",
 	})
 
+	set.EnumSingleVar(&flag.EnumSingleVar{
+		Name:    "nomad-service-provider",
+		Target:  &i.config.serviceProvider,
+		Usage:   "Create service for Waypoint UI and Server.",
+		Values:  []string{"consul", "nomad", "none"},
+		Default: "none", // default none on upgrades, make sure people opt-in to service discovery
+	})
+
+	set.StringSliceVar(&flag.StringSliceVar{
+		Name:    "nomad-service-ui-tags",
+		Target:  &i.config.serviceUITags,
+		Usage:   "Tags for the Waypoint UI service.",
+		Default: []string{defaultServiceTag},
+	})
+
+	set.StringSliceVar(&flag.StringSliceVar{
+		Name:    "nomad-service-backend-tags",
+		Target:  &i.config.serviceBackendTags,
+		Usage:   "Tags for the Waypoint backend service.",
+		Default: []string{defaultServiceTag},
+	})
+
 	set.BoolVar(&flag.BoolVar{
 		Name:    "nomad-consul-service",
 		Target:  &i.config.consulService,
@@ -1438,10 +1543,9 @@ func (i *NomadInstaller) UpgradeFlags(set *flag.Set) {
 	})
 
 	set.StringSliceVar(&flag.StringSliceVar{
-		Name:    "nomad-consul-service-ui-tags",
-		Target:  &i.config.consulServiceUITags,
-		Usage:   "Tags for the Waypoint UI service generated in Consul.",
-		Default: []string{defaultConsulServiceTag},
+		Name:   "nomad-consul-service-ui-tags",
+		Target: &i.config.consulServiceUITags,
+		Usage:  "Tags for the Waypoint UI service generated in Consul.",
 	})
 
 	set.StringSliceVar(&flag.StringSliceVar{
@@ -1449,7 +1553,6 @@ func (i *NomadInstaller) UpgradeFlags(set *flag.Set) {
 		Target: &i.config.consulServiceBackendTags,
 		Usage: "Tags for the Waypoint backend service generated in Consul. The 'first' tag " +
 			"will be used when crafting the Consul DNS hostname for accessing Waypoint.",
-		Default: []string{defaultConsulServiceTag},
 	})
 
 	set.StringVar(&flag.StringVar{
