@@ -6,6 +6,8 @@ import (
 	"strings"
 	"time"
 
+	"github.com/hashicorp/waypoint/pkg/server/hcerr"
+
 	"github.com/hashicorp/go-hclog"
 	"github.com/hashicorp/go-memdb"
 
@@ -14,6 +16,7 @@ import (
 	"google.golang.org/grpc/status"
 	"google.golang.org/protobuf/types/known/emptypb"
 	empty "google.golang.org/protobuf/types/known/emptypb"
+	"google.golang.org/protobuf/types/known/timestamppb"
 
 	"github.com/hashicorp/waypoint/internal/telemetry/metrics"
 	pb "github.com/hashicorp/waypoint/pkg/server/gen"
@@ -28,7 +31,7 @@ func (s *Service) ListRunners(
 ) (*pb.ListRunnersResponse, error) {
 	runners, err := s.state(ctx).RunnerList()
 	if err != nil {
-		return nil, err
+		return nil, hcerr.Externalize(hclog.FromContext(ctx), err, "failed to list runners")
 	}
 	return &pb.ListRunnersResponse{Runners: runners}, nil
 }
@@ -38,7 +41,11 @@ func (s *Service) GetRunner(
 	ctx context.Context,
 	req *pb.GetRunnerRequest,
 ) (*pb.Runner, error) {
-	return s.state(ctx).RunnerById(req.RunnerId, nil)
+	result, err := s.state(ctx).RunnerById(req.RunnerId, nil)
+	if err != nil {
+		return nil, hcerr.Externalize(hclog.FromContext(ctx), err, "failed to get runner", "id", req.RunnerId)
+	}
+	return result, err
 }
 
 func (s *Service) RunnerGetDeploymentConfig(
@@ -81,30 +88,36 @@ func (s *Service) AdoptRunner(
 	ctx context.Context,
 	req *pb.AdoptRunnerRequest,
 ) (*empty.Empty, error) {
-	if err := serverptypes.ValidateAdoptRunnerRequest(req); err != nil {
+	var err error
+	if err = serverptypes.ValidateAdoptRunnerRequest(req); err != nil {
 		return nil, err
 	}
-
-	var err error
+	log := hclog.FromContext(ctx)
 	if req.Adopt {
-		err = s.state(ctx).RunnerAdopt(req.RunnerId, false)
+		if err = s.state(ctx).RunnerAdopt(req.RunnerId, false); err != nil {
+			return &empty.Empty{}, hcerr.Externalize(log, err, "failed to adopt runner", "id", req.RunnerId)
+		}
 	} else {
-		err = s.state(ctx).RunnerReject(req.RunnerId)
+		if err = s.state(ctx).RunnerReject(req.RunnerId); err != nil {
+			return &empty.Empty{}, hcerr.Externalize(log, err, "failed to reject runner", "id", req.RunnerId)
+		}
 	}
 
-	return &empty.Empty{}, err
+	return &empty.Empty{}, nil
 }
 
 func (s *Service) ForgetRunner(
 	ctx context.Context,
 	req *pb.ForgetRunnerRequest,
 ) (*empty.Empty, error) {
-	if err := serverptypes.ValidateForgetRunnerRequest(req); err != nil {
+	var err error
+	if err = serverptypes.ValidateForgetRunnerRequest(req); err != nil {
 		return nil, err
 	}
-
-	err := s.state(ctx).RunnerDelete(req.RunnerId)
-	return &empty.Empty{}, err
+	if err = s.state(ctx).RunnerDelete(req.RunnerId); err != nil {
+		return &empty.Empty{}, hcerr.Externalize(hclog.FromContext(ctx), err, "failed to delete runner", "id", req.RunnerId)
+	}
+	return &empty.Empty{}, nil
 }
 
 func (s *Service) RunnerToken(
@@ -116,7 +129,7 @@ func (s *Service) RunnerToken(
 
 	// Get our token because our behavior changes a bit with different tokens.
 	// Token may be nil because this is an unauthenticated endpoint.
-	if tok := s.tokenFromContext(ctx); tok != nil {
+	if tok := s.decodedTokenFromContext(ctx); tok != nil {
 		switch k := tok.Kind.(type) {
 		case *pb.Token_Login_:
 			// Legacy (pre WP 0.8) token. We accept these as preadopted. We just
@@ -178,7 +191,7 @@ func (s *Service) RunnerToken(
 	log = log.With("runner_id", record.Id)
 	log.Trace("registering runner")
 	if err := s.state(ctx).RunnerCreate(record); err != nil {
-		return nil, err
+		return nil, hcerr.Externalize(log, err, "failed to create runner", "id", record.Id)
 	}
 
 	// When we exit, mark the runner as offline. This will delete the record
@@ -197,7 +210,7 @@ func (s *Service) RunnerToken(
 		r = nil
 	}
 	if err != nil {
-		return nil, err
+		return nil, hcerr.Externalize(log, err, "unknown runner connected", "id", record.Id)
 	}
 	prevAdopted := r != nil && r.AdoptionState == pb.Runner_ADOPTED
 
@@ -216,7 +229,7 @@ func (s *Service) RunnerToken(
 		ws := memdb.NewWatchSet()
 		r, err := s.state(ctx).RunnerById(record.Id, ws)
 		if err != nil {
-			return nil, err
+			return nil, hcerr.Externalize(log, err, "failed to get runner while waiting for adoption state to change", "id", record.Id)
 		}
 
 		switch r.AdoptionState {
@@ -244,7 +257,7 @@ func (s *Service) RunnerToken(
 				// expire and introduce rotation as a feature of adoption.
 				0,
 
-				DefaultKeyId,
+				s.activeAuthKeyId,
 				nil,
 				&pb.Token{
 					Kind: &pb.Token_Runner_{
@@ -298,7 +311,7 @@ func (s *Service) RunnerConfig(
 	log = log.With("runner_id", record.Id)
 	log.Trace("registering runner")
 	if err := s.state(ctx).RunnerCreate(record); err != nil {
-		return err
+		return hcerr.Externalize(log, err, "failed to create runner", "id", record.Id)
 	}
 
 	// Mark the runner as offline if they disconnect from the config stream loop.
@@ -313,7 +326,7 @@ func (s *Service) RunnerConfig(
 	// do not allow it to continue, even with a preadoption token.
 	r, err := s.state(ctx).RunnerById(record.Id, nil)
 	if err != nil {
-		return err
+		return hcerr.Externalize(log, err, "failed to get newly-registered runner", "id", record.Id)
 	}
 	if r.AdoptionState == pb.Runner_REJECTED {
 		return status.Errorf(codes.PermissionDenied,
@@ -321,7 +334,7 @@ func (s *Service) RunnerConfig(
 	}
 	if r.AdoptionState != pb.Runner_ADOPTED {
 		if err := s.state(ctx).RunnerAdopt(record.Id, true); err != nil {
-			return err
+			return hcerr.Externalize(log, err, "failed to adopt runner", "id", record.Id)
 		}
 	}
 
@@ -355,7 +368,7 @@ func (s *Service) RunnerConfig(
 		// Get a job assignment for this runner, non-blocking
 		sjob, err := s.state(ctx).JobPeekForRunner(ctx, record)
 		if err != nil {
-			return err
+			return hcerr.Externalize(log, err, "failed to get job for runner", "id", record.Id)
 		}
 		if sjob == nil {
 			return status.Errorf(codes.FailedPrecondition,
@@ -366,6 +379,20 @@ func (s *Service) RunnerConfig(
 		// Set our job
 		job = sjob.Job
 
+		// We know a job was accepted, so it shouldn't be hanging around because this runner
+		// is available.
+		log.Trace("updating expiry time for job to be 60 seconds now that runner has been assigned job")
+		dur, err := time.ParseDuration("60s")
+		if err != nil {
+			return status.Errorf(codes.FailedPrecondition,
+				"Invalid expiry duration: %s", err.Error())
+		}
+
+		newExpireTime := timestamppb.New(time.Now().Add(dur))
+		if err := s.state(ctx).JobUpdateExpiry(job.Id, newExpireTime); err != nil {
+			return hcerr.Externalize(log, err, "failed to update job expiry time after runner accepted job", "id", job.Id)
+		}
+
 		log.Debug("runner is scoped for config",
 			"project/application", job.Application,
 			"workspace", job.Workspace,
@@ -397,7 +424,7 @@ func (s *Service) RunnerConfig(
 
 		vars, err := s.state(ctx).ConfigGetWatch(configReq, ws)
 		if err != nil {
-			return err
+			return hcerr.Externalize(log, err, "failed to get configuration variables")
 		}
 		config.ConfigVars = vars
 
@@ -418,7 +445,7 @@ func (s *Service) RunnerConfig(
 				},
 			}, ws)
 			if err != nil {
-				return err
+				return hcerr.Externalize(log, err, "failed to get the configuration for a dynamic source plugin")
 			}
 
 			config.ConfigSources = sources
@@ -465,8 +492,7 @@ func (s *Service) RunnerJobStream(
 
 	runner, err := s.state(ctx).RunnerById(runnerId, nil)
 	if err != nil {
-		log.Error("unknown runner connected", "id", runnerId)
-		return err
+		return hcerr.Externalize(log, err, "failed to get this runner", "id", runnerId)
 	}
 	log.With("runner-id", runner.Id)
 
@@ -492,7 +518,7 @@ func (s *Service) RunnerJobStream(
 		log.Info("runner reattaching to an existing job", "job_id", jobId)
 		job, err = s.state(ctx).JobById(jobId, nil)
 		if err != nil {
-			return err
+			return hcerr.Externalize(log, err, "failed to get job", "id", jobId)
 		}
 
 		// If the job is not found, that is an error.
@@ -517,7 +543,7 @@ func (s *Service) RunnerJobStream(
 		log.Info("waiting for job assignment")
 		job, err = s.state(ctx).JobAssignForRunner(ctx, runner)
 		if err != nil {
-			return err
+			return hcerr.Externalize(log, err, "failed to get job assignment for runner", "id", runnerId)
 		}
 	}
 	if job == nil || job.Job == nil {
@@ -532,8 +558,7 @@ func (s *Service) RunnerJobStream(
 		},
 	}, nil)
 	if err != nil {
-		log.Warn("failed to load config sourcers for job assignment", "err", err)
-		return err
+		return hcerr.Externalize(log, err, "failed to get the configuration for a dynamic source plugin to send with job assignment")
 	}
 	log.Trace("loaded config sources for job", "total_sourcers", len(cfgSrcs))
 
@@ -599,6 +624,10 @@ func (s *Service) RunnerJobStream(
 			// If this fails, we just log, there is nothing more we can do.
 			log.Warn("job ack failed", "outer_error", err, "error", ackerr)
 
+			// Check if job is nil, so not to panic later on
+			if job == nil {
+				return hcerr.Externalize(log, ackerr, "job is nil, db might not be open")
+			}
 			// If we had no outer error, set the ackerr so that we exit. If
 			// we do have an outer error, then the ack error only shows up in
 			// the log.
@@ -619,8 +648,12 @@ func (s *Service) RunnerJobStream(
 
 	// If we have an error, return that. We also return if we didn't ack for
 	// any reason. This error can be set at any point since job assignment.
-	if err != nil || !ack {
-		return err
+	if err != nil {
+		return hcerr.Externalize(log, err, "failed to ack the job or the job was cancelled", "id", runnerId, "job", job.Id)
+	}
+	if !ack {
+		// If runners don't ack the job, this means close the stream
+		return nil
 	}
 
 	var logStreamWriter logstream.Writer
@@ -725,6 +758,7 @@ func (s *Service) RunnerJobStream(
 				select {
 				case req := <-eventCh:
 					if err := s.handleJobStreamRequest(log, job, server, req, logStreamWriter); err != nil {
+						log.Error("error handling job stream request during drain", "err", err, "req", req)
 						return err
 					}
 				default:
@@ -733,10 +767,12 @@ func (s *Service) RunnerJobStream(
 			}
 
 		case err := <-errCh:
+			log.Error("err from err channel", "err", err)
 			return err
 
 		case req := <-eventCh:
 			if err := s.handleJobStreamRequest(log, job, server, req, logStreamWriter); err != nil {
+				log.Error("error handling job stream request", "err", err, "req", req)
 				return err
 			}
 
@@ -750,6 +786,8 @@ func (s *Service) RunnerJobStream(
 			// cancel requests are made.
 			if job.CancelTime != nil &&
 				(lastJob == nil || !lastJob.CancelTime.AsTime().Equal(job.CancelTime.AsTime())) {
+				log.Trace("job cancellation request received")
+
 				// The job is forced if we're in an error state. This must be true
 				// because we would've already exited the loop if we naturally
 				// got a terminal event.
@@ -763,6 +801,7 @@ func (s *Service) RunnerJobStream(
 					},
 				})
 				if err != nil {
+					log.Error("error sending job cancel event to runner", "err", err)
 					return err
 				}
 
@@ -772,6 +811,7 @@ func (s *Service) RunnerJobStream(
 				}
 			}
 
+			log.Trace("updating job from state store", "last_job", lastJob, "job", job.Job)
 			lastJob = job.Job
 		}
 	}
@@ -788,34 +828,43 @@ func (s *Service) handleJobStreamRequest(
 	log.Trace("event received", "event", req.Event)
 	switch event := req.Event.(type) {
 	case *pb.RunnerJobStreamRequest_Complete_:
-		return s.state(ctx).JobComplete(job.Id, event.Complete.Result, nil)
-
+		if err := s.state(ctx).JobComplete(job.Id, event.Complete.Result, nil); err != nil {
+			return hcerr.Externalize(log, err, "failed to complete job", "id", job.Id)
+		}
 	case *pb.RunnerJobStreamRequest_Error_:
-		return s.state(ctx).JobComplete(job.Id, nil, status.FromProto(event.Error.Error).Err())
-
+		if err := s.state(ctx).JobComplete(job.Id, nil, status.FromProto(event.Error.Error).Err()); err != nil {
+			return hcerr.Externalize(log, err, "failed to complete job", "id", job.Id)
+		}
 	case *pb.RunnerJobStreamRequest_Heartbeat_:
-		return s.state(ctx).JobHeartbeat(job.Id)
-
+		if err := s.state(ctx).JobHeartbeat(job.Id); err != nil {
+			return hcerr.Externalize(log, err, "job heartbeat failed", "id", job.Id)
+		}
 	case *pb.RunnerJobStreamRequest_Download:
 		if err := s.state(ctx).JobUpdateRef(job.Id, event.Download.DataSourceRef); err != nil {
-			return err
+			return hcerr.Externalize(log, err, "failed to update the job reference", "id", job.Id)
 		}
 
-		return s.state(ctx).ProjectUpdateDataRef(&pb.Ref_Project{
+		if err := s.state(ctx).ProjectUpdateDataRef(&pb.Ref_Project{
 			Project: job.Application.Project,
-		}, job.Workspace, event.Download.DataSourceRef)
+		}, job.Workspace, event.Download.DataSourceRef); err != nil {
+			return hcerr.Externalize(log, err, "failed to update the project", "project", job.Application.Project)
+		}
 
 	case *pb.RunnerJobStreamRequest_ConfigLoad_:
-		return s.state(ctx).JobUpdate(job.Id, func(jobpb *pb.Job) error {
+		if err := s.state(ctx).JobUpdate(job.Id, func(jobpb *pb.Job) error {
 			jobpb.Config = event.ConfigLoad.Config
 			return nil
-		})
+		}); err != nil {
+			return hcerr.Externalize(log, err, "failed to update the job with config", "id", job.Id)
+		}
 
 	case *pb.RunnerJobStreamRequest_VariableValuesSet_:
-		return s.state(ctx).JobUpdate(job.Id, func(jobpb *pb.Job) error {
+		if err := s.state(ctx).JobUpdate(job.Id, func(jobpb *pb.Job) error {
 			jobpb.VariableFinalValues = event.VariableValuesSet.FinalValues
 			return nil
-		})
+		}); err != nil {
+			return hcerr.Externalize(log, err, "failed to update the job with variables", "id", job.Id)
+		}
 
 	case *pb.RunnerJobStreamRequest_Terminal:
 		// Write the events
@@ -839,7 +888,7 @@ func (s *Service) runnerVerifyToken(
 	runnerLabels map[string]string, // real runner labels
 ) error {
 	// Get our token and reverify that we are adopted.
-	tok := s.tokenFromContext(ctx)
+	tok := s.decodedTokenFromContext(ctx)
 	if tok == nil {
 		log.Error("no token, should not be possible")
 		return status.Errorf(codes.Unauthenticated, "no token")
