@@ -1,7 +1,7 @@
 import Route from '@ember/routing/route';
 
 export default class OnboardingIndex extends Route {
-  redirect() {
-    return this.transitionTo('onboarding.install');
+  redirect(): void {
+    this.transitionTo('onboarding.install');
   }
 }
