@@ -3,8 +3,8 @@ import classic from 'ember-classic-decorator';
 
 @classic
 class WaypointSessionService extends SessionService {
-  handleInvalidation(...args): void {
-    super.handleInvalidation(...args);
+  handleInvalidation(routeAfterInvalidation: string): void {
+    super.handleInvalidation(routeAfterInvalidation);
     this.set('data.workspace', undefined);
   }
 }
